#!/usr/bin/perl -w
#
# batchPHYML.pl v1.0
# Last modified March 3, 2006 14:40
# (c) Olaf R.P. Bininda-Emonds
#
# Purpose:
#	Will conduct sequential PHYML analyzes on a set of data matrices specfied by the user in a tab-delimited text file
#
# Input:
#   A tab-delimited text file of the form data set name<tab>model of evolution
#
# Output:
#   Cleaned DNA sequence data (via seqCleaner.pl) in any of fasta, nexus or Se-Al formats.
#	Will preserve name and accession number associations of sequences.
#
# Requires:
#   phyml in the $PATH (which can be specified in line xxx or using the switch -p)
#
# Usage: batchPHYML.pl -d<filename> [-b<number>] [-p<name>] [-v] [-h]
#	options: -b<number> = number of bootstrap replicates to be performed (default = 1000; use 0 to turn off option)
#            -d<filename> = tab-delimited file of the form data set name<tab>model of evolution
#            -p<name> = name (and optionally path) to phyml
#            -h = print this message and quit
#            -v = verbose output

use strict;

# Set user-input defaults and associated parameters
	# I/O variables
		my $dataFiles;
		my (@seqFiles, %optModel);

	# PHYML variables
		my $phyml = "phyml";
 		my %modelCmd;
 		my $bootReps = 1000;
 		my $bootCmd = "B\n1000\nY\n";

	# Miscellaneous variables
		my $verbose = 0;
		my $debug = 0;
		my $version = "1.0";

# Process flags
	for (my $i = 0; $i <= $#ARGV; $i++)
		{
		if ($ARGV[$i] =~ /^-b(\d+)/)
			{
			$bootReps = $1;
			$bootCmd =~ s/1000/$bootReps/ if ($bootReps);
			}
		elsif ($ARGV[$i] =~ /^-d(.*)/)
			{
			$dataFiles = $1;
			die "ERROR: File containing names of data sets to be analyzed $dataFiles does not exist.\n" unless (-e $dataFiles);
			}
		elsif ($ARGV[$i] =~ /^-p(.+)/)
			{
			$phyml = $1;
			}
		elsif ($ARGV[$i] eq "-v")
			{
			$verbose = 1;
			}
		elsif ($ARGV[$i] eq "-x")
			{
			$debug = 1;
			$verbose = 1;
			}
		elsif ($ARGV[$i] eq "-h")
			{
			print "Usage: batchPHYML.pl -d<filename> [-b<number>] [-p<name>] [-v] [-h]\n";
			print "Version: $version\n";
			print "Options: -b<number> = number of bootstrap replicates to be performed (default = 1000; use 0 to turn off option)\n";
			print "         -d<filename> = tab-delimited file of the form data set name<tab>model of evolution\n";
			print "         -p<name> = name (and optionally path) to phyml\n";
			print "         -v = verbose output\n";
			print "         -h = print this message and quit\n";
			exit(0);
			}
		else
			{
			print "Don't understand argument: $ARGV[$i]\n";
			print "Usage: batchPHYML.pl -d<filename> [-b<number>] [-p<name>] [-v] [-h]\n";
			print "Version: $version\n";
			exit(1); 
			}
		}

# Read in files to be analyzed and optimal models
	setLineBreak($dataFiles);
	open (FILES, "<$dataFiles") or die "Cannot open file containing names of sequence files to be analyzed, $dataFiles\n";
		while (<FILES>)
			{
			chomp;
			next unless ($_);
			next if ($_ =~ /^Data file\tModel/);	# Ignore header line from autoMT.pl output
			next if ($_ =~ /^Note:/);	# Ignore footer line from autoMT.pl output
			
			my @fileLine = split(/\t/, $_);
			push @seqFiles, $fileLine[0];
			$optModel{$fileLine[0]} = $fileLine[1];
				$optModel{$fileLine[0]} =~ s/\s+\*$//;	# Get rid of trailing asterisk potentially added by autoMT.pl
			}
	close FILES;
	
	die "ERROR: No files read in from $dataFiles.\n" if (not @seqFiles);

# Load PHYML command into memory
	&PHYMLmodels;
	
# Analyze each data set in turn
	foreach my $dataSet (@seqFiles)
		{
		print "Analyzing data set $dataSet ...\n";
			print "\tModel of evolution: $optModel{$dataSet}\n";
			print "\tNumber of bootstrap replicates: $bootReps\n";

		my $seqFile;
			if (-e $dataSet)
				{
				$seqFile = $dataSet;
				}
			else
				{
				$seqFile = $dataSet . ".phylip";
					$seqFile =~ s/\.phylip\.phylip$/\.phylip/;
				}
			unless (-e $seqFile)
				{
				print "\n\tERROR: Cannot find associated file for data set $dataSet ($seqFile); skipping to next data set.\n\n";
				next;
				}
		my $resultFile = $dataSet . "_phyml_log.txt";
			
		# Create command statement and print to file
			my $phymlCmd;
				$phymlCmd .= $bootCmd if ($bootReps > 1);
				if (defined $modelCmd{$optModel{$dataSet}})
					{
					$phymlCmd .= $modelCmd{$optModel{$dataSet}};
					}
				else
					{
					print "\n\tERROR: Model for data set $dataSet, $optModel{$dataSet}, not recognized; skipping to next data set.\n\n";
					next;
					}
					
			open (CMD, ">phyml_commands.txt") or die "Cannot write to PHYML command file $!\n";
				print CMD "$seqFile\n$phymlCmd";
			close CMD;
			
		# Analyze data set
			my $phymlZero = time;
			
			# Remove associated PHYML output files to avoid overwrite warnings
				unlink ("${dataSet}_phyml_lk.txt", "${dataSet}_phyml_stat.txt", "${dataSet}_phyml_boot_stats.txt", "${dataSet}_phyml_tree.txt", "${dataSet}_phyml_boot_trees.txt", "${dataSet}_phyml_log.txt");

			system ("$phyml < phyml_commands.txt > $resultFile");
				print "\n\tResults written to $resultFile\n";
			
			my $phymlTime = time - $phymlZero;
				if ($phymlTime > 3600)
					{
					printf "\t\tTime taken: %.1f hours\n\n", $phymlTime / 3600;
					}
				else
					{
					print "\t\tTime taken: $phymlTime seconds\n\n";
					}
		}
	unlink ("phyml_commands.txt");

# Subroutines used in the script

sub setLineBreak	# Check line breaks of input files and set input record separator accordingly
	{
	my $inFile = shift;
	$/ ="\n";
	open (IN, "<$inFile") or die "Cannot open $inFile to check form of line breaks.\n";
		while (<IN>)
			{
			if ($_ =~ /\r\n/)
				{
				print "DOS line breaks detected ...\n" if ($verbose);
				$/ ="\r\n";
				last;
				}
			elsif ($_ =~ /\r/)
				{
				print "Mac line breaks detected ...\n" if ($verbose);
				$/ ="\r";
				last;
				}
			else
				{
				print "Unix line breaks detected ...\n" if ($verbose);
				$/ ="\n";
				last;
				}
			}
	close IN;
	}

sub PHYMLmodels	# Idea modeled on solution used in MrAIC.pl by Johan Nylander
	{
	$modelCmd{"JC69"}	=	"M\nM\nM\nM\nM\nY\n";	# JC69
	$modelCmd{"JC69+I"}	=	"M\nM\nM\nM\nM\nV\nY\nY\n";	# JC69I
	$modelCmd{"JC69+G"}	=	"M\nM\nM\nM\nM\nR\nA\nY\nY\n";	# JC69G
	$modelCmd{"JC69+I+G"}	=	"M\nM\nM\nM\nM\nR\nA\nY\nV\nY\nY\n";	# JC69IG
	$modelCmd{"F81"}	=	"M\nM\nM\nM\nM\nM\nM\nY\n";	# F81
	$modelCmd{"F81+I"}	=	"M\nM\nM\nM\nM\nM\nM\nV\nY\nY\n";	# F81I
	$modelCmd{"F81+G"}	=	"M\nM\nM\nM\nM\nM\nM\nR\nA\nY\nY\n";	# F81G
	$modelCmd{"F81+I+G"}	=	"M\nM\nM\nM\nM\nM\nM\nR\nA\nY\nV\nY\nY\n";	# F81IG
	$modelCmd{"K80"}	=	"M\nM\nM\nM\nM\nM\nT\nY\nY\n";	# K80
	$modelCmd{"K80+I"}	=	"M\nM\nM\nM\nM\nM\nT\nY\nV\nY\nY\n";	# K80I
	$modelCmd{"K80+G"}	=	"M\nM\nM\nM\nM\nM\nT\nY\nR\nA\nY\nY\n";	# K80G
	$modelCmd{"K80+I+G"}	=	"M\nM\nM\nM\nM\nM\nT\nY\nR\nA\nY\nV\nY\nY\n";	# K80IG
	$modelCmd{"HKY"}	=	"T\nY\nE\nY\n";	# HKY
	$modelCmd{"HKY+I"}	=	"T\nY\nE\nV\nY\nY\n";	# HKYI
	$modelCmd{"HKY+G"}	=	"T\nY\nE\nR\nA\nY\nY\n";	# HKYG
	$modelCmd{"HKY+I+G"}	=	"T\nY\nE\nR\nA\nY\nV\nY\nY\n";	# HKYIG
	$modelCmd{"TrNef"}	=	"M\nM\nM\nM\nF\n0.25\n0.25\n0.25\n0.25\nK\n010020\n1.00\n1.00\nW\nY\n";	# TrNef
	$modelCmd{"TrNef+I"}	=	"M\nM\nM\nM\nF\n0.25\n0.25\n0.25\n0.25\nK\n010020\n1.00\n1.00\nW\nV\nY\nY\n";	# TrNefI
	$modelCmd{"TrNef+G"}	=	"M\nM\nM\nM\nF\n0.25\n0.25\n0.25\n0.25\nK\n010020\n1.00\n1.00\nW\nR\nA\nY\nY\n";	# TrNefG
	$modelCmd{"TrNef+I+G"}	=	"M\nM\nM\nM\nF\n0.25\n0.25\n0.25\n0.25\nK\n010020\n1.00\n1.00\nW\nR\nA\nY\nV\nY\nY\n";	# TrNefIG
	$modelCmd{"TrN"}	=	"M\nM\nE\nT\nY\nY\n";	# TrN
	$modelCmd{"TrN+I"}	=	"M\nM\nE\nT\nY\nV\nY\nY\n";	# TrNI
	$modelCmd{"TrN+G"}	=	"M\nM\nE\nT\nY\nR\nA\nY\nY\n";	# TrNG
	$modelCmd{"TrN+I+G"}	=	"M\nM\nE\nT\nY\nR\nA\nY\nV\nY\nY\n";	# TrNIG
	$modelCmd{"K3P"}	=	"M\nM\nM\nM\nF\n0.25\n0.25\n0.25\n0.25\nK\n012210\n1.00\n1.00\nW\nY\n";	# K3P
	$modelCmd{"K3P+I"}	=	"M\nM\nM\nM\nF\n0.25\n0.25\n0.25\n0.25\nK\n012210\n1.00\n1.00\nW\nV\nY\nY\n";	# K3PI
	$modelCmd{"K3P+G"}	=	"M\nM\nM\nM\nF\n0.25\n0.25\n0.25\n0.25\nK\n012210\n1.00\n1.00\nW\nR\nA\nY\nY\n";	# K3PG
	$modelCmd{"K3P+I+G"}	=	"M\nM\nM\nM\nF\n0.25\n0.25\n0.25\n0.25\nK\n012210\n1.00\n1.00\nW\nR\nA\nY\nV\nY\nY\n";	# K3PIG
	$modelCmd{"K3Puf"}	=	"M\nM\nM\nM\nK\n012210\n1.00\n1.00\nW\nY\n";	# K3Puf
	$modelCmd{"K3Puf+I"}	=	"M\nM\nM\nM\nK\n012210\n1.00\n1.00\nW\nV\nY\nY\n";	# K3PufI
	$modelCmd{"K3Puf+G"}	=	"M\nM\nM\nM\nK\n012210\n1.00\n1.00\nW\nR\nA\nY\nY\n";	# K3PufG
	$modelCmd{"K3Puf+I+G"}	=	"M\nM\nM\nM\nK\n012210\n1.00\n1.00\nW\nR\nA\nY\nV\nY\nY\n";	# K3PufIG
	$modelCmd{"TIMef"}	=	"M\nM\nM\nM\nF\n0.25\n0.25\n0.25\n0.25\nK\n012230\n1.00\n1.00\n1.00\nW\nY\n";	# TIMef
	$modelCmd{"TIMef+I"}	=	"M\nM\nM\nM\nF\n0.25\n0.25\n0.25\n0.25\nK\n012230\n1.00\n1.00\n1.00\nW\nV\nY\nY\n";	# TIMefI
	$modelCmd{"TIMef+G"}	=	"M\nM\nM\nM\nF\n0.25\n0.25\n0.25\n0.25\nK\n012230\n1.00\n1.00\n1.00\nW\nR\nA\nY\nY\n";	# TIMefG
	$modelCmd{"TIMef+I+G"}	=	"M\nM\nM\nM\nF\n0.25\n0.25\n0.25\n0.25\nK\n012230\n1.00\n1.00\n1.00\nW\nR\nA\nY\nV\nY\nY\n";	# TIMefIG
	$modelCmd{"TIM"}	=	"M\nM\nM\nM\nK\n012230\n1.00\n1.00\n1.00\nW\nY\n";	# TIM
	$modelCmd{"TIM+I"}	=	"M\nM\nM\nM\nK\n012230\n1.00\n1.00\n1.00\nW\nV\nY\nY\n";	# TIMI
	$modelCmd{"TIM+G"}	=	"M\nM\nM\nM\nK\n012230\n1.00\n1.00\n1.00\nW\nR\nA\nY\nY\n";	# TIMG
	$modelCmd{"TIM+I+G"}	=	"M\nM\nM\nM\nK\n012230\n1.00\n1.00\n1.00\nW\nR\nA\nY\nV\nY\nY\n";	# TIMIG
	$modelCmd{"TVMef"}	=	"M\nM\nM\nM\nF\n0.25\n0.25\n0.25\n0.25\nK\n012314\n1.00\n1.00\n1.00\n1.00\nW\nY\n";	# TVMef
	$modelCmd{"TVMef+I"}	=	"M\nM\nM\nM\nF\n0.25\n0.25\n0.25\n0.25\nK\n012314\n1.00\n1.00\n1.00\n1.00\nW\nV\nY\nY\n";	# TVMefI
	$modelCmd{"TVMef+G"}	=	"M\nM\nM\nM\nF\n0.25\n0.25\n0.25\n0.25\nK\n012314\n1.00\n1.00\n1.00\n1.00\nW\nR\nA\nY\nY\n";	# TVMefG
	$modelCmd{"TVMef+I+G"}	=	"M\nM\nM\nM\nF\n0.25\n0.25\n0.25\n0.25\nK\n012314\n1.00\n1.00\n1.00\n1.00\nW\nR\nA\nY\nV\nY\nY\n";	# TVMefIG
	$modelCmd{"TVM"}	=	"M\nM\nM\nM\nK\n012314\n1.00\n1.00\n1.00\n1.00\nW\nY\n";	# TVM
	$modelCmd{"TVM+I"}	=	"M\nM\nM\nM\nK\n012314\n1.00\n1.00\n1.00\n1.00\nW\nV\nY\nY\n";	# TVMI
	$modelCmd{"TVM+G"}	=	"M\nM\nM\nM\nK\n012314\n1.00\n1.00\n1.00\n1.00\nW\nR\nA\nY\nY\n";	# TVMG
	$modelCmd{"TVM+I+G"}	=	"M\nM\nM\nM\nK\n012314\n1.00\n1.00\n1.00\n1.00\nW\nR\nA\nY\nV\nY\nY\n";	# TVMIG
	$modelCmd{"SYM"}	=	"M\nM\nM\nM\nF\n0.25\n0.25\n0.25\n0.25\nK\n012345\n1.00\n1.00\n1.00\n1.00\n1.00\nW\nY\n";	# SYM
	$modelCmd{"SYM+I"}	=	"M\nM\nM\nM\nF\n0.25\n0.25\n0.25\n0.25\nK\n012345\n1.00\n1.00\n1.00\n1.00\n1.00\nW\nV\nY\nY\n";	# SYMI
	$modelCmd{"SYM+G"}	=	"M\nM\nM\nM\nF\n0.25\n0.25\n0.25\n0.25\nK\n012345\n1.00\n1.00\n1.00\n1.00\n1.00\nW\nR\nA\nY\nY\n";	# SYMG
	$modelCmd{"SYM+I+G"}	=	"M\nM\nM\nM\nF\n0.25\n0.25\n0.25\n0.25\nK\n012345\n1.00\n1.00\n1.00\n1.00\n1.00\nW\nR\nA\nY\nV\nY\nY\n";	# SYMIG
	$modelCmd{"GTR"}	=	"M\nM\nM\nE\nY\n";	# GTR
	$modelCmd{"GTR+I"}	=	"M\nM\nM\nE\nV\nY\nY\n";	# GTRI
	$modelCmd{"GTR+G"}	=	"M\nM\nM\nE\nR\nA\nY\nY\n";	# GTRG
	$modelCmd{"GTR+I+G"}	=	"M\nM\nM\nE\nR\nA\nY\nV\nY\nY\n";	# GTRIG
	}

# Version history
#
# 	v1.0 (March 3, 2006)
#		- initial release
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# A copy of the GNU General Public License is available at
# http://www.gnu.org/copyleft/gpl.html or by writing to
# the Free Software Foundation, Inc., 51 Franklin Street,
# Fifth Floor, Boston, MA, 02110-1301, USA.
